@@ -710,16 +710,27 @@ void MvRequest :: addValueToAll(const char* reqname, const char* parname, double
 }
 #endif
 
-int MvRequest::countParameters() const
+int MvRequest::countParameters(bool includeHidden) const
 {
     if (!CurrentRequest)
         return 0;
     int n        = 0;
     parameter* p = CurrentRequest->params;
+    parameter* lastp = p;
     while (p) {
         n++;
+        lastp = p;
         p = p->next;
     }
+    // here, 'hidden' means that it starts with a double underscore, as in "__strings"
+    if (!includeHidden) {
+        if (lastp &&
+            lastp->name &&
+            strlen(lastp->name) > 1 &&
+            lastp->name[0] == '_' &&
+            lastp->name[1] == '_')
+            n--;
+    }
     return n;
 }
 
