@@ -3,55 +3,37 @@
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ *
  * In applying this licence, ECMWF does not waive the privileges and immunities
  * granted to it by virtue of its status as an intergovernmental organisation nor
  * does it submit to any jurisdiction.
  */
 
-/// @author Tiago Quintino
-/// @author Pedro Maciel
-/// @author Baudouin Raoult
-/// @author Willem Deconinck
-/// @date May 2015
-
 
 #include "mir/method/fe/FiniteElement.h"
 
 #include <algorithm>
+#include <cmath>
 #include <forward_list>
+#include <iterator>
 #include <limits>
 #include <map>
 #include <memory>
+#include <mutex>
+#include <ostream>
+#include <sstream>
 #include <utility>
 
-#include "eckit/config/Resource.h"
 #include "eckit/log/ResourceUsage.h"
 #include "eckit/log/TraceTimer.h"
-#include "eckit/thread/AutoLock.h"
-#include "eckit/thread/Mutex.h"
-#include "eckit/types/FloatCompare.h"
 #include "eckit/utils/MD5.h"
 
-#include "atlas/interpolation/element/Quad3D.h"
-#include "atlas/interpolation/element/Triag3D.h"
-#include "atlas/interpolation/method/PointIndex3.h"
-#include "atlas/interpolation/method/Ray.h"
-#include "atlas/mesh/Elements.h"
-#include "atlas/mesh/Nodes.h"
-#include "atlas/mesh/actions/BuildCellCentres.h"
-#include "atlas/mesh/actions/BuildNode2CellConnectivity.h"
-#include "atlas/mesh/actions/BuildXYZField.h"
-#include "atlas/output/Gmsh.h"
-
-#include "mir/caching/InMemoryCache.h"
+#include "mir/api/Atlas.h"
+#include "mir/caching/InMemoryMeshCache.h"
 #include "mir/config/LibMir.h"
-#include "mir/method/fe/BuildNodeLumpedMassMatrix.h"
-#include "mir/method/fe/CalculateCellLongestDiagonal.h"
-#include "mir/param/MIRParametrisation.h"
 #include "mir/repres/Iterator.h"
 #include "mir/repres/Representation.h"
 #include "mir/util/Domain.h"
-#include "mir/util/MIRStatistics.h"
 #include "mir/util/Pretty.h"
 
 
@@ -60,16 +42,20 @@ namespace method {
 namespace fe {
 
 
-namespace {
-
-
 // epsilon used to scale edge tolerance when projecting ray to intesect element
 static const double parametricEpsilon = 1e-15;
 
+static pthread_once_t once                             = PTHREAD_ONCE_INIT;
+static std::mutex* mtx                                 = nullptr;
+static std::map<std::string, FiniteElementFactory*>* m = nullptr;
+static void init() {
+    mtx = new std::mutex();
+    m   = new std::map<std::string, FiniteElementFactory*>();
+}
 
-using triplet_vector_t = std::vector< WeightMatrix::Triplet >;
-using element_tree_t = atlas::interpolation::method::ElemIndex3;
-using failed_projection_t = std::pair< size_t, PointLatLon >;
+using triplet_vector_t    = std::vector<WeightMatrix::Triplet>;
+using element_tree_t      = atlas::interpolation::method::ElemIndex3;
+using failed_projection_t = std::pair<size_t, PointLatLon>;
 
 
 static void normalise(triplet_vector_t& triplets) {
@@ -81,36 +67,30 @@ static void normalise(triplet_vector_t& triplets) {
         sum += t.value();
     }
 
+    // normalise all weights according to the total
     if (sum > std::numeric_limits<double>::epsilon()) {
-
-        // now normalise all weights according to the total
         const double invSum = 1. / sum;
         for (auto& t : triplets) {
             t.value() *= invSum;
         }
+        return;
+    }
 
-    } else {
-
-        // if no reasonable seight sum is found, distribute equitably
-        const double invSum = 1. / triplets.size();
-        for (auto& t : triplets) {
-            t.value() = invSum;
-        }
-
+    // if no reasonable weight sum is found, distribute equitably
+    const double invSum = 1. / triplets.size();
+    for (auto& t : triplets) {
+        t.value() = invSum;
     }
 }
 
 
 /// Find in which element the point is contained by projecting the point with each nearest element
-static triplet_vector_t projectPointTo3DElements(
-    size_t nbInputPoints,
-    const atlas::array::ArrayView<double, 2>& icoords,
-    const atlas::mesh::HybridElements::Connectivity& connectivity,
-    const Point3& p,
-    size_t ip,
-    size_t firstVirtualPoint,
-    size_t& nbProjectionAttempts,
-    const element_tree_t::NodeList& closest ) {
+static triplet_vector_t projectPointTo3DElements(size_t nbInputPoints,
+                                                 const atlas::array::ArrayView<double, 2>& icoords,
+                                                 const atlas::mesh::HybridElements::Connectivity& connectivity,
+                                                 const Point3& p, size_t ip, size_t firstVirtualPoint,
+                                                 size_t& nbProjectionAttempts,
+                                                 const element_tree_t::NodeList& closest) {
 
     if (closest.empty()) {
         return {};
@@ -121,7 +101,11 @@ static triplet_vector_t projectPointTo3DElements(
     bool mustNormalise = false;
     size_t idx[4];
     double w[4];
-    atlas::interpolation::method::Ray ray( p.data() );
+    constexpr size_t XX = 0;
+    constexpr size_t YY = 1;
+    constexpr size_t ZZ = 2;
+
+    atlas::interpolation::method::Ray ray(p.data());
 
     nbProjectionAttempts = 0;
     for (const auto& close : closest) {
@@ -147,9 +131,9 @@ static triplet_vector_t projectPointTo3DElements(
 
             /* triangle */
             atlas::interpolation::element::Triag3D triag(
-                atlas::PointXYZ{ icoords(idx[0], 0), icoords(idx[0], 1), icoords(idx[0], 2) },
-                atlas::PointXYZ{ icoords(idx[1], 0), icoords(idx[1], 1), icoords(idx[1], 2) },
-                atlas::PointXYZ{ icoords(idx[2], 0), icoords(idx[2], 1), icoords(idx[2], 2) });
+                atlas::PointXYZ{icoords(idx[0], XX), icoords(idx[0], YY), icoords(idx[0], ZZ)},
+                atlas::PointXYZ{icoords(idx[1], XX), icoords(idx[1], YY), icoords(idx[1], ZZ)},
+                atlas::PointXYZ{icoords(idx[2], XX), icoords(idx[2], YY), icoords(idx[2], ZZ)});
 
             // pick an epsilon based on a characteristic length (sqrt(area))
             // (this scales linearly so it better compares with linear weights u,v,w)
@@ -164,8 +148,7 @@ static triplet_vector_t projectPointTo3DElements(
                 w[1] = is.u;
                 w[2] = is.v;
 
-                for (size_t i = 0; i < 3; ++i)
-                {
+                for (size_t i = 0; i < 3; ++i) {
                     if (idx[i] < firstVirtualPoint) {
                         triplets.push_back(WeightMatrix::Triplet(ip, idx[i], w[i]));
                     }
@@ -174,19 +157,19 @@ static triplet_vector_t projectPointTo3DElements(
                     }
                 }
 
-                break; // stop looking for elements
+                break;  // stop looking for elements
             }
-
-        } else {
+        }
+        else {
 
             /* quadrilateral */
             atlas::interpolation::element::Quad3D quad(
-                atlas::PointXYZ{ icoords(idx[0], 0), icoords(idx[0], 1), icoords(idx[0], 2) },
-                atlas::PointXYZ{ icoords(idx[1], 0), icoords(idx[1], 1), icoords(idx[1], 2) },
-                atlas::PointXYZ{ icoords(idx[2], 0), icoords(idx[2], 1), icoords(idx[2], 2) },
-                atlas::PointXYZ{ icoords(idx[3], 0), icoords(idx[3], 1), icoords(idx[3], 2) });
+                atlas::PointXYZ{icoords(idx[0], XX), icoords(idx[0], YY), icoords(idx[0], ZZ)},
+                atlas::PointXYZ{icoords(idx[1], XX), icoords(idx[1], YY), icoords(idx[1], ZZ)},
+                atlas::PointXYZ{icoords(idx[2], XX), icoords(idx[2], YY), icoords(idx[2], ZZ)},
+                atlas::PointXYZ{icoords(idx[3], XX), icoords(idx[3], YY), icoords(idx[3], ZZ)});
 
-            if ( !quad.validate() ) { // somewhat expensive sanity check
+            if (!quad.validate()) {  // somewhat expensive sanity check
                 eckit::Log::warning() << "Invalid Quad : " << quad << std::endl;
                 throw eckit::SeriousBug("Found invalid quadrilateral in mesh", Here());
             }
@@ -201,9 +184,9 @@ static triplet_vector_t projectPointTo3DElements(
 
                 // weights are the bilinear Lagrange function evaluated at u,v
                 w[0] = (1. - is.u) * (1. - is.v);
-                w[1] =       is.u  * (1. - is.v);
-                w[2] =       is.u  *       is.v ;
-                w[3] = (1. - is.u) *       is.v ;
+                w[1] = is.u * (1. - is.v);
+                w[2] = is.u * is.v;
+                w[3] = (1. - is.u) * is.v;
 
 
                 for (size_t i = 0; i < 4; ++i) {
@@ -215,11 +198,11 @@ static triplet_vector_t projectPointTo3DElements(
                     }
                 }
 
-                break; // stop looking for elements
+                break;  // stop looking for elements
             }
         }
 
-    } // loop over nearest elements
+    }  // loop over nearest elements
 
     // at least one of the nodes of element shouldn't be virtual
     if (!triplets.empty() && mustNormalise) {
@@ -230,30 +213,8 @@ static triplet_vector_t projectPointTo3DElements(
 }
 
 
-static caching::InMemoryCache<atlas::Mesh> mesh_cache(
-    "mirMesh",
-    512 * 1024 * 1024,
-    0,
-    "$MIR_MESH_CACHE_MEMORY_FOOTPRINT" );
-
-
-static pthread_once_t once                             = PTHREAD_ONCE_INIT;
-static eckit::Mutex* local_mutex                       = nullptr;
-static std::map<std::string, FiniteElementFactory*>* m = nullptr;
-
-
-static void init() {
-    local_mutex = new eckit::Mutex();
-    m           = new std::map<std::string, FiniteElementFactory*>();
-}
-
-
-}  // (anonymous namespace)
-
-
 FiniteElement::FiniteElement(const param::MIRParametrisation& param, const std::string& label) :
-    MethodWeighted(param),
-    meshGeneratorParams_(label, param) {
+    MethodWeighted(param), meshGeneratorParams_(label, param) {
 
     // input mesh requirements
     meshGeneratorParams_.meshCellCentres_ = true;
@@ -271,7 +232,7 @@ atlas::Mesh FiniteElement::atlasMesh(util::MIRStatistics& statistics, const repr
         params.meshCellLongestDiagonal_ = true;
     }
 
-    auto msh = atlasMesh(statistics, repres.atlasGrid(), params);
+    auto msh = caching::InMemoryMeshCache::atlasMesh(statistics, repres.atlasGrid(), params);
     if (!params.meshCellLongestDiagonal_) {
         ASSERT(d > 0.);
         msh.metadata().set("cell_longest_diagonal", d);
@@ -280,117 +241,10 @@ atlas::Mesh FiniteElement::atlasMesh(util::MIRStatistics& statistics, const repr
 }
 
 
-atlas::Mesh FiniteElement::atlasMesh(util::MIRStatistics& statistics, const atlas::Grid& grid,
-                                     const util::MeshGeneratorParameters& meshGeneratorParams) const {
-    pthread_once(&once, init);
-    eckit::AutoLock<eckit::Mutex> lock(local_mutex);
-
-    eckit::Channel& log = eckit::Log::debug<LibMir>();
-
-    eckit::ResourceUsage usage_mesh("Mesh for grid " + grid.name() + " (" + grid.uid() + ")", log);
-    caching::InMemoryCacheUser<atlas::Mesh> cache_use(mesh_cache, statistics.meshCache_);
-
-    // generate signature including the mesh generation settings
-    eckit::MD5 md5;
-    md5 << grid;
-    md5 << meshGeneratorParams;
-
-    auto j = mesh_cache.find(md5);
-    if (j != mesh_cache.end()) {
-        return *j;
-    }
-
-    atlas::Mesh& mesh = mesh_cache[md5];
-    ASSERT(!mesh.generated());
-
-    try {
-
-        const std::string meshGenerator = meshGeneratorParams.meshGenerator_;
-        if (meshGenerator.empty()) {
-            throw eckit::SeriousBug("Mesh: no mesh generator defined ('" + meshGenerator + "')");
-        }
-
-        atlas::MeshGenerator generator(meshGenerator, meshGeneratorParams);
-        mesh = generator.generate(grid);
-        ASSERT(mesh.generated());
-
-        // If meshgenerator did not create xyz field already, do it now.
-        {
-            eckit::ResourceUsage usage("BuildXYZField", log);
-            eckit::TraceTimer<LibMir> timer("Mesh: BuildXYZField");
-            atlas::mesh::actions::BuildXYZField()(mesh);
-        }
-
-        // Calculate barycenters of mesh cells
-        if (meshGeneratorParams.meshCellCentres_) {
-            eckit::ResourceUsage usage("BuildCellCentres", log);
-            eckit::TraceTimer<LibMir> timer("Mesh: BuildCellCentres");
-            atlas::mesh::actions::BuildCellCentres()(mesh);
-        }
-
-        // Calculate the mesh cells longest diagonal
-        if (meshGeneratorParams.meshCellLongestDiagonal_) {
-            eckit::ResourceUsage usage("CalculateCellLongestDiagonal", log);
-            eckit::TraceTimer<LibMir> timer("Mesh: CalculateCellLongestDiagonal");
-            CalculateCellLongestDiagonal()(mesh);
-        }
-
-        // Calculate node-lumped mass matrix
-        if (meshGeneratorParams.meshNodeLumpedMassMatrix_) {
-            eckit::ResourceUsage usage("BuildNodeLumpedMassMatrix", log);
-            eckit::TraceTimer<LibMir> timer("Mesh: BuildNodeLumpedMassMatrix");
-            BuildNodeLumpedMassMatrix()(mesh);
-        }
-
-        // Calculate node-to-cell ("inverse") connectivity
-        if (meshGeneratorParams.meshNodeToCellConnectivity_) {
-            eckit::ResourceUsage usage("BuildNode2CellConnectivity", log);
-            eckit::TraceTimer<LibMir> timer("Mesh: BuildNode2CellConnectivity");
-            atlas::mesh::actions::BuildNode2CellConnectivity{mesh}();
-        }
-
-        // Some information
-        log << "Mesh["
-               "cells="
-            << Pretty(mesh.cells().size()) << ",nodes=" << Pretty(mesh.nodes().size()) << ","
-            << meshGeneratorParams << "]" << std::endl;
-
-        // Write file(s)
-        if (!meshGeneratorParams.fileLonLat_.empty()) {
-            atlas::output::PathName path(meshGeneratorParams.fileLonLat_);
-            log << "Mesh: writing to '" << path << "'" << std::endl;
-            atlas::output::Gmsh(path, atlas::util::Config("coordinates", "lonlat")).write(mesh);
-        }
-
-        if (!meshGeneratorParams.fileXY_.empty()) {
-            atlas::output::PathName path(meshGeneratorParams.fileXY_);
-            log << "Mesh: writing to '" << path << "'" << std::endl;
-            atlas::output::Gmsh(path, atlas::util::Config("coordinates", "xy")).write(mesh);
-        }
-
-        if (!meshGeneratorParams.fileXYZ_.empty()) {
-            atlas::output::PathName path(meshGeneratorParams.fileXYZ_);
-            log << "Mesh: writing to '" << path << "'" << std::endl;
-            atlas::output::Gmsh(path, atlas::util::Config("coordinates", "xyz")).write(mesh);
-        }
-    }
-    catch (...) {
-        // Make sure we don't leave an incomplete entry in the cache
-        mesh_cache.erase(md5);
-        throw;
-    }
-
-    mesh_cache.footprint(md5, caching::InMemoryCacheUsage(mesh.footprint(), 0));
-
-    ASSERT(mesh.generated());
-    return mesh;
-}
-
-
 FiniteElement::~FiniteElement() = default;
 
 
-void FiniteElement::print(std::ostream &out) const {
+void FiniteElement::print(std::ostream& out) const {
     out << "FiniteElement[method=" << name() << ",";
     MethodWeighted::print(out);
     out << "]";
@@ -399,9 +253,7 @@ void FiniteElement::print(std::ostream &out) const {
 
 bool FiniteElement::sameAs(const Method& other) const {
     auto o = dynamic_cast<const FiniteElement*>(&other);
-    return o
-            && meshGeneratorParams_.sameAs(o->meshGeneratorParams_)
-            && MethodWeighted::sameAs(other);
+    return (o != nullptr) && meshGeneratorParams_.sameAs(o->meshGeneratorParams_) && MethodWeighted::sameAs(other);
 }
 
 
@@ -411,9 +263,7 @@ void FiniteElement::hash(eckit::MD5& md5) const {
 }
 
 
-void FiniteElement::assemble(util::MIRStatistics& statistics,
-                             WeightMatrix& W,
-                             const repres::Representation& in,
+void FiniteElement::assemble(util::MIRStatistics& statistics, WeightMatrix& W, const repres::Representation& in,
                              const repres::Representation& out) const {
     eckit::Channel& log = eckit::Log::debug<LibMir>();
 
@@ -423,11 +273,11 @@ void FiniteElement::assemble(util::MIRStatistics& statistics,
     // get input mesh
     ASSERT(meshGeneratorParams().meshCellCentres_);  // required for the k-d tree
 
-    const atlas::Mesh& inMesh = atlasMesh(statistics, in);
+    const atlas::Mesh& inMesh    = atlasMesh(statistics, in);
     const util::Domain& inDomain = in.domain();
 
     const atlas::mesh::Nodes& inNodes = inMesh.nodes();
-    atlas::array::ArrayView<double, 2> icoords = atlas::array::make_view< double, 2 >( inNodes.field( "xyz" ));
+    auto icoords                      = atlas::array::make_view<double, 2>(inNodes.field("xyz"));
 
     size_t firstVirtualPoint = std::numeric_limits<size_t>::max();
     if (inNodes.metadata().has("NbRealPts")) {
@@ -462,8 +312,8 @@ void FiniteElement::assemble(util::MIRStatistics& statistics,
 
 
     // weights -- one per vertex of element, triangles (3) or quads (4)
-    triplet_vector_t weights_triplets; // structure to fill-in sparse matrix
-    weights_triplets.reserve( nbOutputPoints * 4 );        // preallocate space as if all elements where quads
+    triplet_vector_t weights_triplets;             // structure to fill-in sparse matrix
+    weights_triplets.reserve(nbOutputPoints * 4);  // preallocate space as if all elements where quads
 
     {
         Pretty::ProgressTimer progress("Projecting", nbOutputPoints, {"point"}, log);
@@ -488,25 +338,19 @@ void FiniteElement::assemble(util::MIRStatistics& statistics,
                 element_tree_t::NodeList closest = eTree->findInSphere(p, R);
 
                 size_t nbProjectionAttempts = 0;
-                triplet_vector_t triplets = projectPointTo3DElements(
-                                                nbInputPoints,
-                                                icoords,
-                                                connectivity,
-                                                p,
-                                                ip,
-                                                firstVirtualPoint,
-                                                nbProjectionAttempts,
-                                                closest );
+                triplet_vector_t triplets   = projectPointTo3DElements(nbInputPoints, icoords, connectivity, p, ip,
+                                                                     firstVirtualPoint, nbProjectionAttempts, closest);
 
-                nbMaxElementsSearched = std::max(nbMaxElementsSearched, closest.size());
-                nbMinElementsSearched = std::min(nbMinElementsSearched, closest.size());
+                nbMaxElementsSearched   = std::max(nbMaxElementsSearched, closest.size());
+                nbMinElementsSearched   = std::min(nbMinElementsSearched, closest.size());
                 nbMaxProjectionAttempts = std::max(nbMaxProjectionAttempts, nbProjectionAttempts);
 
                 if (triplets.empty()) {
                     // If this fails, consider lowering parametricEpsilon
                     failures.push_front(failed_projection_t(ip, it->pointUnrotated()));
                     ++nbFailures;
-                } else {
+                }
+                else {
                     std::copy(triplets.begin(), triplets.end(), std::back_inserter(weights_triplets));
                     ++nbProjections;
                 }
@@ -522,7 +366,7 @@ void FiniteElement::assemble(util::MIRStatistics& statistics,
         << Pretty(nbMaxElementsSearched, {"element"}) << ", with up to "
         << Pretty(nbMaxProjectionAttempts, {"projection attempt"}) << " (per point)" << std::endl;
 
-    if (nbFailures && !failuresAreMissingValues) {
+    if ((nbFailures > 0) && !failuresAreMissingValues) {
         std::stringstream msg;
         msg << "Failed to project " << Pretty(nbFailures, {"point"});
         log << msg.str() << ":";
@@ -544,12 +388,9 @@ void FiniteElement::assemble(util::MIRStatistics& statistics,
 }
 
 
-//=========================================================================
-
-
 FiniteElementFactory::FiniteElementFactory(const std::string& name) : MethodFactory(name), name_(name) {
     pthread_once(&once, init);
-    eckit::AutoLock<eckit::Mutex> lock(local_mutex);
+    std::lock_guard<std::mutex> guard(*mtx);
 
     if (m->find(name) != m->end()) {
         throw eckit::SeriousBug("FiniteElementFactory: duplicate '" + name + "'");
@@ -561,7 +402,8 @@ FiniteElementFactory::FiniteElementFactory(const std::string& name) : MethodFact
 
 
 FiniteElementFactory::~FiniteElementFactory() {
-    eckit::AutoLock<eckit::Mutex> lock(local_mutex);
+    pthread_once(&once, init);
+    std::lock_guard<std::mutex> guard(*mtx);
 
     m->erase(name_);
 }
@@ -569,7 +411,7 @@ FiniteElementFactory::~FiniteElementFactory() {
 
 void FiniteElementFactory::list(std::ostream& out) {
     pthread_once(&once, init);
-    eckit::AutoLock<eckit::Mutex> lock(local_mutex);
+    std::lock_guard<std::mutex> guard(*mtx);
 
     const char* sep = "";
     for (const auto& j : *m) {
@@ -582,7 +424,7 @@ void FiniteElementFactory::list(std::ostream& out) {
 FiniteElement* FiniteElementFactory::build(const std::string& method, const std::string& label,
                                            const param::MIRParametrisation& param) {
     pthread_once(&once, init);
-    eckit::AutoLock<eckit::Mutex> lock(local_mutex);
+    std::lock_guard<std::mutex> guard(*mtx);
 
     eckit::Log::debug<LibMir>() << "FiniteElementFactory: looking for '" << method << "'" << std::endl;
 
